@ECHO OFF
setlocal enableDelayedExpansion
TITLE EXALUX
ECHO -----------------------------------------------------------------
ECHO                         EXALUX - WELCOME
ECHO -----------------------------------------------------------------
ECHO.
ECHO This software will force the transfer of ALL the DMX profiles to 
ECHO your CONTROL ONE, without checking the already loaded files.
ECHO -----------------------------------------------------------------
ECHO.
ECHO.
ECHO STEP 1
ECHO   Connect your CONTROL ONE to your computer via micro-USB.
ECHO   Turn on the device.
ECHO. 
ECHO.
ECHO.
ECHO STEP 2
ECHO   Wait for Windows to automatilly install the required drivers 
ECHO   (this can take a while)
ECHO   or install them manually from the DRIVERS folder provided.
ECHO.
ECHO.
ECHO.
ECHO STEP 3
ECHO   Once drivers are installed, press any key to continue ...
PAUSE >nul



::-----------------------------------------------------------------------------
:: Get the path of the current folder, and create the variables that contain the other useful path
SET mypath=%~dp0\..\

:: Full path to the utility folder
SET utilityPath=%mypath%utility

:: Text file, containing a list of all the files stored on the Control One
SET filesondisplayRAW=%mypath%utility\filesondisplayRAW.txt
SET filesondisplay=%mypath%utility\filesondisplay.txt


:: Full path, to where the local files to send are stored. 
:: Two folders are used :
:: > sources, where all the files are provided by Exalux
:: > filestotransfer, that will be used by FileTransfer.exe To make sure the files are not corrupted, it's better to make a copy of the files.
SET sourcesPath=%mypath%sources
SET filetotransferPath=%mypath%filestotransfer

:: Text file, containing a list of all the files stored on the local folders. Only the ones that are not up to date on the Control One will be used
SET fileoncomputer=%mypath%utility\fileoncomputer.txt

:: Text file, containing a list of all the files that need to be transfered to the Control One.
:: Another file is used to store alphabetically all the files
SET filestosend=%mypath%utility\filestosend.txt
SET filestosendSorted=%mypath%utility\filestosendSorted.txt
SET nbfilestosend=0
SET totalsizefilestosend=0

:: Keep track of the number of .x files (only used to display info to the user)
SET nbdmxfiles=0


::-----------------------------------------------------------------------------
:: Delete the temporary files that might already exist
:: (remaining from previous usage of this batch)
CD "%utilityPath%"
DEL *.txt 2>nul


::-----------------------------------------------------------------------------
:: Count the number of COM port that look like CONTROL ONE/SKY.
:: (Use a temporary file to store the result of the commands. Easier than variables)
wmic path win32_pnpentity get caption | FIND "Silicon Labs CP210x USB to UART Bridge" /c > tmpFile.txt
SET /p nbComFound= < tmpFile.txt
DEL tmpFile.txt




::-----------------------------------------------------------------------------
:: If there is only 1 COM Port that corresponds to the CONTROL ONE/SKY,
:: Extract the COM Port number, and only ask for confirmation.
:: NOTE : In the IF/ELSE, make sure to use ! and not %, because of the Expension Delay
SET askUserToSelectComPort=0
IF "%nbComFound%" equ "1" (
	
	CLS
	ECHO -----------------------------------------------------------------
	ECHO                              EXALUX
	ECHO -----------------------------------------------------------------
	ECHO.
	ECHO.
	
	:: Only 1 COM Port found. The following line extracts the number, in the format "COMxx"
	FOR /f "tokens=2delims=()" %%A IN ('wmic path win32_pnpentity get caption /format:table^| FIND "Silicon Labs CP210x USB to UART Bridge"') DO SET "COM=%%A"

	:: Remove the "COM", just to keep the number
	SET myCom=!COM:~3!
	
	:: Ask for confirmation to the user
	ECHO The software detected a CONTROL ONE on COM!myCom!.
	ECHO.
	ECHO.
	ECHO To use COM!myCom!, press ENTER.
	SET /p userChoice="To select another COM Port, press N. "
	
	IF /I "!userChoice!" == "n" ( 
		SET askUserToSelectComPort=1
	)	
)

IF "%nbComFound%" NEQ "1" (
	:: There are multiple COM ports -> Ask the user
	SET askUserToSelectComPort=1
)



::-----------------------------------------------------------------------------
:: If several COM Port correspond to the CONTROL ONE/SKY, or if the user
:: chose to select the COM Port himself, show the list of available COM Ports
IF "%askUserToSelectComPort%" == "1" (

	CLS
	ECHO -----------------------------------------------------------------
	ECHO                              EXALUX
	ECHO -----------------------------------------------------------------
	ECHO.
	ECHO.
	ECHO Please, select the COM Port corresponding to your CONTROL ONE.
	ECHO The CONTROL ONE/SKY is detected as "Silicon Labs CP210x".
	ECHO.
	ECHO.
	ECHO The software detected the following COM Ports :
	ECHO.
	ECHO *********************************************************
	ECHO.
	wmic path win32_pnpentity get caption /format:table | FIND "COM"
	ECHO *********************************************************
	ECHO.
	ECHO.	
	ECHO|SET /p="Enter the COM Port number to use : "
	SET /p myCom="COM"
)
ECHO.
ECHO.
ECHO.
ECHO Processing ...




::-----------------------------------------------------------------------------
:: Kill any previous FileTransfer.exe that might be still running 
:: (2>nul to not show the error in case FileTransfer is not already running)
TASKKILL /IM FileTransfer.exe /F 2>nul




::-----------------------------------------------------------------------------
:: Start checking all the files present on the SD card of the display
:: In case some files already exist on the SD, and have not been updated since last library, there is no need to re-load them on the SD.
cls
ECHO -----------------------------------------------------------------
ECHO                              EXALUX
ECHO -----------------------------------------------------------------
ECHO.
ECHO The CONTROL ONE will reboot several times during the process.
ECHO.
ECHO DO NOT UNPLUG OR POWER OFF THE CONTROL ONE.
ECHO.
ECHO.
ECHO Please wait ...
ECHO.


::-----------------------------------------------------------------------------
:: Delete the filestotransfer folder.
:: It will be re-generated a bit later, with fresh files from "sources"
rmdir "%filetotransferPath%" /s /q 2>nul

::-----------------------------------------------------------------------------
:: Create a fresh "filestotransfer" folder, from a copy of "sources" folder
:: This is important, because in case FileTransfer fails, the files are sometimes corrupted.
:: By using a copied folder, we are sure to always have intact files, even if FileTransfer failled previously.
xcopy "%sourcesPath%" "%filetotransferPath%"  /i /q 1>nul



::-----------------------------------------------------------------------------
:: List all the files that are ready to send on the PC.
:: There are should only be 3 types of files
::	- X files -> DMX profile in the "profile" folder
::	- EXA -> LIBIND (can also be a memory file, but not used here)
::	- 4FN -> Firmware update for any side-app (keyboard, ...)
::
:: The list should be formated in the same way the files on display is formated: "FILE_NAME"<space>FILE_SIZE<space>FILE_DATE
ECHO _ Collecting files on computer ...
CD "%filetotransferPath%"

:: Both FORFILES are needed, to keep the batch working with the existing code.
:: One is used directly to have the list of files. The other is for verification.
FORFILES /C "CMD /C ECHO @path">>"%filestosend%"
FORFILES /C "CMD /C ECHO @file @fsize @fdate">>"%fileoncomputer%"

:: Count the number of files 
FOR %%I IN (*.X) DO SET /A nbfilestosend=nbfilestosend+1



:: From here, the file %filestosend% contains the list of all the files that should be transfered to the CONTROL ONE

::-----------------------------------------------------------------------------
:: Check if the CONTROL ONE is up to date : if it 
IF %nbfilestosend% == 0 (
	CLS
	COLOR 0A
	ECHO -----------------------------------------------------------------
	ECHO                              EXALUX
	ECHO -----------------------------------------------------------------
	ECHO.
	ECHO.
	ECHO No files were found on the computer.
	ECHO Make sure that the downloaded folder is not corrupted.
	ECHO.
	ECHO NO UPDATE HAS BEEN PERFORMED ^^!
	ECHO.
	ECHO.
	COLOR
	PAUSE
	EXIT
)

::-----------------------------------------------------------------------------
:: Remove the 1st line of the file (Command FORFILE always returns a first empty line)
:: Then, sort reverse-alphabetically the files : we want LIBINDEX to be transfered at the end
:: For these actions, a temporary file is used.
more +1 "%filestosend%" > myTmp.tmp
SORT myTmp.tmp>>"%filestosendSorted%"
DEL myTmp.tmp



::-----------------------------------------------------------------------------
:: Compute the number of bytes that are going to be transfered
SET totalsizefilestosend=0
FOR /f "usebackq tokens=* delims=" %%A IN ("%filestosendSorted%") DO (
	SET /A totalsizefilestosend=totalsizefilestosend+%%~zA
)




::-----------------------------------------------------------------------------
:: Start the transfer to the C_ONE.	
:: The computation of the time estimation is empirical : average throughput is 105kbps 
:: Add an extra 10 seconds, to ensure that the CONTROL ONE has rebooted from the initial call of FileTransfer
CLS
ECHO -----------------------------------------------------------------
ECHO                              EXALUX
ECHO -----------------------------------------------------------------
ECHO.
SET /A dur=10 + %totalsizefilestosend% * 10 / 105000
SET /A durMin=%dur%/60
SET /A durSec=%dur%-(%durMin%*60)
SET /A totalsizefilestosend=totalsizefilestosend/1024
ECHO Preparing transfer for %nbfilestosend% files (%totalsizefilestosend% kB) to the CONTROL ONE on COM%myCom%.
ECHO.
ECHO Estimated duration : %durMin% min %durSec% s
ECHO|SET /P="Start time: "
%TIME/T
ECHO.
ECHO.
ECHO Your CONTROL ONE will reboot several times during the process.
ECHO.
ECHO _ DO NOT UNPLUG OR POWER OFF THE CONTROL ONE ^^!
ECHO _ DO NOT CLOSE THIS WINDOW ^^!
ECHO.
ECHO.
ECHO Transferring ...
ECHO Files are displayed on the CONTROL ONE's screen. Please wait ...

:: Make sure that Control one has rebooted after the initial call of FileTransfer.exe
TIMEOUT /nobreak 8 >nul



::-----------------------------------------------------------------------------
:: Jump to the utility folder, and launch FileTransfer (and wait for completion)
CD "%utilityPath%"
START /w FileTransfer $"%filestosendSorted%" /b115200 /cCOM%myCom% /Q /s



::-----------------------------------------------------------------------------
:: Verification step : read the content on the Control One, and check if it matches what has been sent.
ECHO Transfer complete.
ECHO.
ECHO.
ECHO Verification. Please wait ...

:: Make sure that Control one has rebooted after the call of FileTransfer.exe
TIMEOUT /nobreak 8 >nul

:: Go read the files on the CONTROL ONE
CALL :readControlOneFiles

:: Check if the CONTROL ONE contains all the files that are on the PC
SET nbfilestosend=0
FOR /f "usebackq tokens=* delims=" %%A IN ("%fileoncomputer%") DO (
		
	FIND "%%A" "%filesondisplay%" 1>nul 2>nul 
	
	:: If the file is not present on the C_ONE, increment counter
	IF ERRORLEVEL 1 (		
		SET /A nbfilestosend=nbfilestosend+1
	) 
)


::-----------------------------------------------------------------------------
:: Display exit message
CLS
IF %nbfilestosend%==0 (	
	
	COLOR 0A
	ECHO -----------------------------------------------------------------
	ECHO                              EXALUX
	ECHO -----------------------------------------------------------------
	ECHO.
	ECHO Transfer and verification is now complete.
	ECHO.
	ECHO All files have been successfully transfered ^^!
	ECHO.
	ECHO Restart your device and enjoy ^^!
	ECHO.
	ECHO.
	ECHO.
	COLOR	
) ELSE (
	COLOR 0C
	ECHO -----------------------------------------------------------------
	ECHO                              EXALUX
	ECHO -----------------------------------------------------------------
	ECHO.
	ECHO Verification failed ^^!
	ECHO.
	ECHO Your CONTROL ONE has not been fully updated ^^!
	ECHO.
	ECHO Errors might be cause by antivirus blocking the execution of
	ECHO the software Filetransfer.exe.
	ECHO Make sure your antivirus is not blocking the execution.
	ECHO.
	ECHO Restart the update procedure.
	ECHO.
	ECHO.
	ECHO.
	COLOR
)

::-----------------------------------------------------------------------------
:: End of program
PAUSE
EXIT






::*********************************************************************************************************::
:: Function to read the content of the CONTROL ONE.
::*********************************************************************************************************::
:readControlOneFiles
:: Jump into the folder that contains FileTransfer.exe
:: Use FileTransfer to get the list of ALL the files stored on the SD card of th screen
CD "%utilityPath%"
IF EXIST "%filesondisplay%" DEL "%filesondisplay%"
IF EXIST "%filesondisplayRAW%" DEL "%filesondisplayRAW%"
START /w FileTransfer *.* "%filesondisplayRAW%" /b115200 /cCOM%myCom% /F /Q /L

:: FileTransfer sometimes returns an empty file (<100 bytes) : need to ensure the file is good.
:: If it's not, try a second time. In case the file is still empty after the second attempt, exit the program.
SET tstSize=0
FOR %%i IN ("%filesondisplayRAW%") DO SET tstSize=%%~zi

IF %tstSize% LEQ 100 (	
	:: Make sure that Control one has rebooted, and re-call FileTransfer
	TIMEOUT /nobreak 8 >nul
	START /w FileTransfer *.* "%filesondisplayRAW%" /b115200 /cCOM%myCom% /F /Q /L

	SET tstSize=0
	FOR %%i IN ("%filesondisplayRAW%") DO SET tstSize=%%~zi
	
	IF !tstSize! LEQ 100 (
		CLS
		ECHO -----------------------------------------------------------------
		ECHO                              EXALUX
		ECHO -----------------------------------------------------------------
		ECHO.
		ECHO.
		COLOR 0C
		ECHO An error occured ^^!
		ECHO.
		ECHO The software was unable to get the data from the CONTROL ONE.
		ECHO.
		ECHO Your CONTROL ONE has not been updated ^^!
		ECHO.
		ECHO Errors might be cause by antivirus blocking the execution of
		ECHO the software Filetransfer.exe.
		ECHO Make sure your antivirus is not blocking the execution.
		ECHO.
		ECHO Restart the update procedure.
		COLOR
		ECHO.
		
		::-----------------------------------------------------------------------------
		:: End of program
		PAUSE
		EXIT

	)	
)

::-----------------------------------------------------------------------------
:: Format the result, so that we easilly compare it with the other reference file
:: The result provided by FileTranfer is formated with spaces between each data. 
:: However, depending on the file name and size, the number of space is different for each line !
:: Only keep : FILE_NAME" "FILE_SIZE" "FILE_DATE (without time)"
FOR /f "usebackq tokens=1,2,3,4 delims= " %%A IN ("%filesondisplayRAW%") DO (
	SET tmp="%%A" %%B %%C
	ECHO !tmp!>>"%filesondisplay%"
)

EXIT /b
:: End of "function"  readControlOneFiles() ***************************************************************